@@ -356,12 +356,15 @@ enum /*< skip >*/
                         guint      message, \
                         GtkWidget *plug) \
   { \
-    GdkEventClient event; \
+    GdkEventClient  event; \
+    GdkWindow      *window; \
+    gint            result; \
     \
     g_return_if_fail (GTK_IS_PLUG (plug)); \
     g_return_if_fail (XFCE_IS_PANEL_PLUGIN (xpp)); \
     g_return_if_fail (GDK_IS_WINDOW (gtk_widget_get_window (plug))); \
     g_return_if_fail (_xpp_atom != GDK_NONE); \
+    g_return_if_fail (GTK_WIDGET_REALIZED (xpp)); \
     \
     event.type = GDK_CLIENT_EVENT; \
     event.window = gtk_widget_get_window (plug); \
@@ -371,12 +374,15 @@ enum /*< skip >*/
     event.data.s[0] = message; \
     event.data.s[1] = 0; \
     \
+    window = gtk_plug_get_socket_window (GTK_PLUG (plug)); \
+    g_return_if_fail (GDK_IS_WINDOW (window)); \
+    \
     gdk_error_trap_push (); \
-    gdk_event_send_client_message ((GdkEvent *) &event,  \
-        GDK_WINDOW_XID (gtk_plug_get_socket_window (GTK_PLUG (plug)))); \
+    gdk_event_send_client_message ((GdkEvent *) &event, GDK_WINDOW_XID (window)); \
     gdk_flush (); \
-    if (gdk_error_trap_pop () != 0) \
-      g_warning ("Failed to send provider-signal %d", message); \
+    result = gdk_error_trap_pop (); \
+    if (G_UNLIKELY (result != 0)) \
+      g_warning ("Failed to send provider-signal %d: X error code %d", message, result); \
   } \
   \
   static void \
@@ -430,47 +436,6 @@ enum /*< skip >*/
       gtk_main_quit (); \
   } \
   \
-  static void \
-  _xpp_set_colormap (GtkWidget *plug) \
-  { \
-    GdkColormap *colormap = NULL; \
-    GdkScreen   *screen; \
-    gboolean     restore; \
-    \
-    g_return_if_fail (GTK_IS_WIDGET (plug)); \
-    \
-    restore = GTK_WIDGET_REALIZED (plug); \
-    if (restore) \
-      { \
-        gtk_widget_hide (plug); \
-        gtk_widget_unrealize (plug); \
-      } \
-    \
-    screen = gtk_widget_get_screen (plug); \
-    \
-    _xpp_composited = gtk_widget_is_composited (plug); \
-    \
-    if (_xpp_composited) \
-      colormap = gdk_screen_get_rgba_colormap (screen); \
-    \
-    if (colormap == NULL) \
-      { \
-        colormap = gdk_screen_get_rgb_colormap (screen); \
-        _xpp_composited = FALSE; \
-      } \
-    \
-    if (colormap != NULL) \
-      gtk_widget_set_colormap (plug, colormap); \
-    \
-    if (restore) \
-      { \
-        gtk_widget_realize (plug); \
-        gtk_widget_show (plug); \
-      } \
-    \
-    gtk_widget_queue_draw (plug); \
-  } \
-  \
   gint \
   main (gint argc, gchar **argv) \
   { \
@@ -479,6 +444,7 @@ enum /*< skip >*/
     GtkWidget       *xpp; \
     gint             unique_id; \
     GdkNativeWindow  socket_id; \
+    GdkColormap     *colormap = NULL; \
     \
     if (G_UNLIKELY (argc < PLUGIN_ARGV_ARGUMENTS)) \
       { \
@@ -505,16 +471,25 @@ enum /*< skip >*/
     \
     socket_id = strtol (argv[PLUGIN_ARGV_SOCKET_ID], NULL, 0); \
     plug = gtk_plug_new (socket_id); \
+    gtk_widget_set_name (GTK_WIDGET (plug), "XfcePanelWindowExternal"); \
     g_signal_connect (G_OBJECT (plug), "embedded", \
         G_CALLBACK (_xpp_plug_embedded), NULL); \
     g_signal_connect (G_OBJECT (plug), "expose-event", \
         G_CALLBACK (_xpp_expose_event), NULL); \
-    g_signal_connect (G_OBJECT (plug), "composited-changed", \
-        G_CALLBACK (_xpp_set_colormap), NULL); \
     \
     gtk_widget_set_app_paintable (plug, TRUE); \
-    if (gtk_widget_is_composited (plug)) \
-      _xpp_set_colormap (plug); \
+    \
+    screen = gtk_widget_get_screen (plug); \
+    _xpp_composited = gtk_widget_is_composited (plug); \
+    if (_xpp_composited) \
+      colormap = gdk_screen_get_rgba_colormap (screen); \
+    if (colormap == NULL) \
+      { \
+        colormap = gdk_screen_get_rgb_colormap (screen); \
+        _xpp_composited = FALSE; \
+      } \
+    if (colormap != NULL) \
+      gtk_widget_set_colormap (plug, colormap); \
     \
     unique_id = strtol (argv[PLUGIN_ARGV_UNIQUE_ID], NULL, 0); \
     xpp = g_object_new (XFCE_TYPE_PANEL_PLUGIN, \
