@@ -129,6 +129,7 @@ struct _PanelApplication
   guint               drop_data_ready : 1;
   guint               drop_occurred : 1;
   guint               drop_desktop_files : 1;
+  guint               drop_index;
 };
 
 enum
@@ -263,6 +264,7 @@ panel_application_xfconf_window_bindings (PanelApplication *application,
     { "horizontal", G_TYPE_BOOLEAN },
     { "size", G_TYPE_UINT },
     { "length", G_TYPE_UINT },
+    { "length-adjust", G_TYPE_BOOLEAN },
     { "enter-opacity", G_TYPE_UINT },
     { "leave-opacity", G_TYPE_UINT },
     { "background-alpha", G_TYPE_UINT },
@@ -716,6 +718,25 @@ panel_application_window_destroyed (GtkWidget        *window,
 
 
 
+static void
+panel_application_windows_block_autohide (PanelApplication *application,
+                                          gboolean          blocked)
+{
+  GSList *li;
+
+  panel_return_if_fail (PANEL_IS_APPLICATION (application));
+
+  for (li = application->windows; li != NULL; li = li->next)
+    {
+      if (blocked)
+        panel_window_freeze_autohide (PANEL_WINDOW (li->data));
+      else
+        panel_window_thaw_autohide (PANEL_WINDOW (li->data));
+    }
+}
+
+
+
 static void
 panel_application_dialog_destroyed (GtkWindow        *dialog,
                                     PanelApplication *application)
@@ -726,6 +747,10 @@ panel_application_dialog_destroyed (GtkWindow        *dialog,
 
   /* remove the window from the list */
   application->dialogs = g_slist_remove (application->dialogs, dialog);
+
+  /* unblock autohide if there are no open windows anymore */
+  if (application->dialogs == NULL)
+    panel_application_windows_block_autohide (application, FALSE);
 }
 
 
@@ -740,7 +765,6 @@ panel_application_drag_data_received (PanelWindow      *window,
                                       guint             drag_time,
                                       GtkWidget        *itembar)
 {
-  guint              position;
   PanelApplication  *application;
   GtkWidget         *provider;
   gboolean           succeed = FALSE;
@@ -792,9 +816,6 @@ panel_application_drag_data_received (PanelWindow      *window,
       /* reset the state */
       application->drop_occurred = FALSE;
 
-      /* get the drop index on the itembar */
-      position = panel_itembar_get_drop_index (PANEL_ITEMBAR (itembar), x, y);
-
       /* get the widget screen */
       screen = gtk_window_get_screen (GTK_WINDOW (window));
 
@@ -807,7 +828,7 @@ panel_application_drag_data_received (PanelWindow      *window,
               name = (const gchar *) selection_data->data;
               succeed = panel_application_plugin_insert (application, window,
                                                          screen, name,
-                                                         -1, NULL, position);
+                                                         -1, NULL, application->drop_index);
             }
           break;
 
@@ -826,12 +847,12 @@ panel_application_drag_data_received (PanelWindow      *window,
                                                             provider);
 
               /* decrease the counter if we drop after the current position */
-              if (position > old_position)
-                position--;
+              if (application->drop_index > old_position)
+                application->drop_index--;
 
               /* reorder the child if needed */
-              if (old_position != position)
-                panel_itembar_reorder_child (PANEL_ITEMBAR (itembar), provider, position);
+              if (old_position != application->drop_index)
+                panel_itembar_reorder_child (PANEL_ITEMBAR (itembar), provider, application->drop_index);
             }
           else
             {
@@ -839,7 +860,7 @@ panel_application_drag_data_received (PanelWindow      *window,
               gtk_widget_reparent (provider, itembar);
 
               /* move the item to the correct position on the itembar */
-              panel_itembar_reorder_child (PANEL_ITEMBAR (itembar), provider, position);
+              panel_itembar_reorder_child (PANEL_ITEMBAR (itembar), provider, application->drop_index);
 
               /* send all the needed panel information to the plugin */
               panel_window_set_povider_info (window, provider);
@@ -860,7 +881,7 @@ panel_application_drag_data_received (PanelWindow      *window,
                   /* create a new item with a unique id */
                   succeed = panel_application_plugin_insert (application, window,
                                                              screen, LAUNCHER_PLUGIN_NAME,
-                                                             -1, uris, position);
+                                                             -1, uris, application->drop_index);
                   g_strfreev (uris);
                 }
 
@@ -946,8 +967,8 @@ panel_application_drag_motion (GtkWidget        *window,
     {
       /* highlight the drop zone */
       itembar = gtk_bin_get_child (GTK_BIN (window));
-      panel_itembar_set_drop_highlight_item (PANEL_ITEMBAR (itembar),
-          panel_itembar_get_drop_index (PANEL_ITEMBAR (itembar), x, y));
+      application->drop_index = panel_itembar_get_drop_index (PANEL_ITEMBAR (itembar), x, y);
+      panel_itembar_set_drop_highlight_item (PANEL_ITEMBAR (itembar), application->drop_index);
     }
 
 not_a_drop_zone:
@@ -1121,11 +1142,13 @@ panel_application_take_dialog (PanelApplication *application,
   panel_return_if_fail (PANEL_IS_APPLICATION (application));
   panel_return_if_fail (GTK_IS_WINDOW (dialog));
 
+  /* block autohide if this will be the first dialog */
+  if (application->dialogs == NULL)
+    panel_application_windows_block_autohide (application, TRUE);
+
   /* monitor window destruction */
   g_signal_connect (G_OBJECT (dialog), "destroy",
       G_CALLBACK (panel_application_dialog_destroyed), application);
-
-  /* add the window to internal list */
   application->dialogs = g_slist_prepend (application->dialogs, dialog);
 }
 
@@ -1351,25 +1374,6 @@ panel_application_windows_sensitive (PanelApplication *application,
 
 
 
-void
-panel_application_windows_autohide (PanelApplication *application,
-                                    gboolean          freeze)
-{
-  GSList *li;
-
-  panel_return_if_fail (PANEL_IS_APPLICATION (application));
-
-  for (li = application->windows; li != NULL; li = li->next)
-    {
-      if (freeze)
-        panel_window_freeze_autohide (PANEL_WINDOW (li->data));
-      else
-        panel_window_thaw_autohide (PANEL_WINDOW (li->data));
-    }
-}
-
-
-
 gboolean
 panel_application_get_locked (PanelApplication *application)
 {
