@@ -68,9 +68,6 @@ void              panel_application_window_select     (PanelApplication  *applic
 void              panel_application_windows_sensitive (PanelApplication  *application,
                                                        gboolean           sensitive);
 
-void              panel_application_windows_autohide  (PanelApplication  *application,
-                                                       gboolean           freeze);
-
 gboolean          panel_application_get_locked        (PanelApplication  *application);
 
 G_END_DECLS
