@@ -59,6 +59,7 @@ static void         panel_plugin_external_46_set_property          (GObject
                                                                     const GValue                     *value,
                                                                     GParamSpec                       *pspec);
 static void         panel_plugin_external_46_realize               (GtkWidget                        *widget);
+static void         panel_plugin_external_46_unrealize             (GtkWidget                        *widget);
 static gboolean     panel_plugin_external_46_client_event          (GtkWidget                        *widget,
                                                                     GdkEventClient                   *event);
 static gboolean     panel_plugin_external_46_plug_removed          (GtkSocket                        *socket);
@@ -85,7 +86,8 @@ static void         panel_plugin_external_46_show_about            (XfcePanelPlu
 static void         panel_plugin_external_46_removed               (XfcePanelPluginProvider          *provider);
 static gboolean     panel_plugin_external_46_remote_event          (XfcePanelPluginProvider          *provider,
                                                                     const gchar                      *name,
-                                                                    const GValue                     *value);
+                                                                    const GValue                     *value,
+                                                                    guint                            *handler_id);
 static void         panel_plugin_external_46_set_locked            (XfcePanelPluginProvider          *provider,
                                                                     gboolean                          locked);
 static void         panel_plugin_external_46_set_sensitive         (PanelPluginExternal46            *external);
@@ -169,6 +171,7 @@ panel_plugin_external_46_class_init (PanelPluginExternal46Class *klass)
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
   gtkwidget_class->realize = panel_plugin_external_46_realize;
+  gtkwidget_class->unrealize = panel_plugin_external_46_unrealize;
   gtkwidget_class->client_event = panel_plugin_external_46_client_event;
 
   gtksocket_class = GTK_SOCKET_CLASS (klass);
@@ -347,11 +350,14 @@ panel_plugin_external_46_realize (GtkWidget *widget)
   guint                   argc = PLUGIN_ARGV_ARGUMENTS;
   guint                   i;
 
+  panel_return_if_fail (external->pid == 0);
+  panel_return_if_fail (!external->plug_embedded);
+
   /* realize the socket first */
   (*GTK_WIDGET_CLASS (panel_plugin_external_46_parent_class)->realize) (widget);
 
   /* get the socket id and unique id in a string */
-  socket_id = g_strdup_printf ("%d", gtk_socket_get_id (GTK_SOCKET (widget)));
+  socket_id = g_strdup_printf ("%u", gtk_socket_get_id (GTK_SOCKET (widget)));
   unique_id = g_strdup_printf ("%d", external->unique_id);
 
   /* add the number of arguments to the argc count */
@@ -405,6 +411,26 @@ panel_plugin_external_46_realize (GtkWidget *widget)
 
 
 
+static void
+panel_plugin_external_46_unrealize (GtkWidget *widget)
+{
+  PanelPluginExternal46  *external = PANEL_PLUGIN_EXTERNAL_46 (widget);
+
+  /* the plug is not embedded anymore */
+  external->plug_embedded = FALSE;
+
+  if (external->watch_id != 0)
+    {
+      /* remove the child watch so we don't leave zomies */
+      g_source_remove (external->watch_id);
+      g_child_watch_add (external->pid, (GChildWatchFunc) g_spawn_close_pid, NULL);
+    }
+
+  (*GTK_WIDGET_CLASS (panel_plugin_external_46_parent_class)->unrealize) (widget);
+}
+
+
+
 static gboolean
 panel_plugin_external_46_client_event (GtkWidget      *widget,
                                        GdkEventClient *event)
@@ -517,6 +543,10 @@ panel_plugin_external_46_plug_added (GtkSocket *socket)
   /* plug has been added */
   external->plug_embedded = TRUE;
 
+  panel_debug (PANEL_DEBUG_DOMAIN_EXTERNAL46,
+               "plugin  %d has been embedded, %d values in queue",
+               external->unique_id, g_slist_length (external->queue));
+
   /* send all the messages in the queue */
   if (external->queue != NULL)
     {
@@ -735,7 +765,8 @@ panel_plugin_external_46_removed (XfcePanelPluginProvider *provider)
 static gboolean
 panel_plugin_external_46_remote_event (XfcePanelPluginProvider *provider,
                                        const gchar             *name,
-                                       const GValue            *value)
+                                       const GValue            *value,
+                                       guint                   *handle)
 {
   panel_return_val_if_fail (PANEL_IS_PLUGIN_EXTERNAL_46 (provider), TRUE);
   panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider), TRUE);
@@ -792,14 +823,24 @@ panel_plugin_external_46_child_watch (GPid     pid,
       switch (WEXITSTATUS (status))
         {
         case PLUGIN_EXIT_SUCCESS:
+          break;
+
         case PLUGIN_EXIT_FAILURE:
         case PLUGIN_EXIT_PREINIT_FAILED:
         case PLUGIN_EXIT_CHECK_FAILED:
         case PLUGIN_EXIT_NO_PROVIDER:
-          /* wait until everything is settled, then destroy the
-           * external plugin so it is removed from the configuration */
+          panel_debug (PANEL_DEBUG_DOMAIN_EXTERNAL46,
+                       "Plugin exited with status %d. Removing from "
+                       "configuration.", WEXITSTATUS (status));
+
+          /* cleanup the plugin configuration (in panel-application) */
+          xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (external),
+                                                  PROVIDER_SIGNAL_REMOVE_PLUGIN);
+
+          /* wait until everything is settled */
           exo_gtk_object_destroy_later (GTK_OBJECT (external));
           break;
+
         }
     }
 
