@@ -43,6 +43,7 @@
 #include <panel/panel-plugin-external.h>
 #include <panel/panel-window.h>
 #include <panel/panel-dialogs.h>
+#include <panel/panel-marshal.h>
 
 
 
@@ -50,63 +51,64 @@
 
 
 
-static void         panel_plugin_external_provider_init         (XfcePanelPluginProviderInterface  *iface);
-static GObject     *panel_plugin_external_constructor           (GType                              type,
-                                                                 guint                              n_construct_params,
-                                                                 GObjectConstructParam             *construct_params);
-static void         panel_plugin_external_finalize              (GObject                           *object);
-static void         panel_plugin_external_get_property          (GObject                           *object,
-                                                                 guint                              prop_id,
-                                                                 GValue                            *value,
-                                                                 GParamSpec                        *pspec);
-static void         panel_plugin_external_set_property          (GObject                           *object,
-                                                                 guint                              prop_id,
-                                                                 const GValue                      *value,
-                                                                 GParamSpec                        *pspec);
-static void         panel_plugin_external_destroy               (GtkObject                         *object);
-static void         panel_plugin_external_realize               (GtkWidget                         *widget);
-static gboolean     panel_plugin_external_plug_removed          (GtkSocket                         *socket);
-static void         panel_plugin_external_plug_added            (GtkSocket                         *socket);
-static gboolean     panel_plugin_external_dbus_reply            (PanelPluginExternal               *external,
-                                                                 guint                              reply_id,
-                                                                 const GValue                      *value,
-                                                                 GError                           **error);
-static gboolean     panel_plugin_external_dbus_provider_signal  (PanelPluginExternal               *external,
-                                                                 XfcePanelPluginProviderSignal      provider_signal,
-                                                                 GError                           **error);
-static void         panel_plugin_external_dbus_set              (PanelPluginExternal               *external,
-                                                                 gboolean                           force);
-static void         panel_plugin_external_queue_add             (PanelPluginExternal               *external,
-                                                                 gboolean                           force,
-                                                                 const gchar                       *property,
-                                                                 const GValue                      *value);
-static void         panel_plugin_external_queue_add_noop        (PanelPluginExternal               *external,
-                                                                 gboolean                           force,
-                                                                 const gchar                       *property);
-static const gchar *panel_plugin_external_get_name              (XfcePanelPluginProvider           *provider);
-static gint         panel_plugin_external_get_unique_id         (XfcePanelPluginProvider           *provider);
-static void         panel_plugin_external_set_size              (XfcePanelPluginProvider           *provider,
-                                                                 gint                               size);
-static void         panel_plugin_external_set_orientation       (XfcePanelPluginProvider           *provider,
-                                                                 GtkOrientation                     orientation);
-static void         panel_plugin_external_set_screen_position   (XfcePanelPluginProvider           *provider,
-                                                                 XfceScreenPosition                 screen_position);
-static void         panel_plugin_external_save                  (XfcePanelPluginProvider           *provider);
-static gboolean     panel_plugin_external_get_show_configure    (XfcePanelPluginProvider           *provider);
-static void         panel_plugin_external_show_configure        (XfcePanelPluginProvider           *provider);
-static gboolean     panel_plugin_external_get_show_about        (XfcePanelPluginProvider           *provider);
-static void         panel_plugin_external_show_about            (XfcePanelPluginProvider           *provider);
-static void         panel_plugin_external_removed               (XfcePanelPluginProvider           *provider);
-static gboolean     panel_plugin_external_remote_event          (XfcePanelPluginProvider           *provider,
-                                                                 const gchar                       *name,
-                                                                 const GValue                      *value);
-static void         panel_plugin_external_set_locked            (XfcePanelPluginProvider           *provider,
-                                                                 gboolean                           locked);
-static void         panel_plugin_external_set_sensitive         (PanelPluginExternal               *external);
-static void         panel_plugin_external_child_watch           (GPid                               pid,
-                                                                 gint                               status,
-                                                                 gpointer                           user_data);
-static void         panel_plugin_external_child_watch_destroyed (gpointer                           user_data);
+static void         panel_plugin_external_provider_init            (XfcePanelPluginProviderInterface  *iface);
+static GObject     *panel_plugin_external_constructor              (GType                              type,
+                                                                    guint                              n_construct_params,
+                                                                    GObjectConstructParam             *construct_params);
+static void         panel_plugin_external_finalize                 (GObject                           *object);
+static void         panel_plugin_external_get_property             (GObject                           *object,
+                                                                    guint                              prop_id,
+                                                                    GValue                            *value,
+                                                                    GParamSpec                        *pspec);
+static void         panel_plugin_external_set_property             (GObject                           *object,
+                                                                    guint                              prop_id,
+                                                                    const GValue                      *value,
+                                                                    GParamSpec                        *pspec);
+static void         panel_plugin_external_realize                  (GtkWidget                         *widget);
+static void         panel_plugin_external_unrealize                (GtkWidget                         *widget);
+static gboolean     panel_plugin_external_plug_removed             (GtkSocket                         *socket);
+static void         panel_plugin_external_plug_added               (GtkSocket                         *socket);
+static gboolean     panel_plugin_external_dbus_provider_signal     (PanelPluginExternal               *external,
+                                                                    XfcePanelPluginProviderSignal      provider_signal,
+                                                                    GError                           **error);
+static gboolean     panel_plugin_external_dbus_remote_event_result (PanelPluginExternal               *external,
+                                                                    guint                              handle,
+                                                                    gboolean                           result,
+                                                                    GError                           **error);
+static void         panel_plugin_external_dbus_set                 (PanelPluginExternal               *external,
+                                                                    gboolean                           force);
+static void         panel_plugin_external_queue_add                (PanelPluginExternal               *external,
+                                                                    gboolean                           force,
+                                                                    const gchar                       *property,
+                                                                    const GValue                      *value);
+static void         panel_plugin_external_queue_add_noop           (PanelPluginExternal               *external,
+                                                                    gboolean                           force,
+                                                                    const gchar                       *property);
+static const gchar *panel_plugin_external_get_name                 (XfcePanelPluginProvider           *provider);
+static gint         panel_plugin_external_get_unique_id            (XfcePanelPluginProvider           *provider);
+static void         panel_plugin_external_set_size                 (XfcePanelPluginProvider           *provider,
+                                                                    gint                               size);
+static void         panel_plugin_external_set_orientation          (XfcePanelPluginProvider           *provider,
+                                                                    GtkOrientation                     orientation);
+static void         panel_plugin_external_set_screen_position      (XfcePanelPluginProvider           *provider,
+                                                                    XfceScreenPosition                 screen_position);
+static void         panel_plugin_external_save                     (XfcePanelPluginProvider           *provider);
+static gboolean     panel_plugin_external_get_show_configure       (XfcePanelPluginProvider           *provider);
+static void         panel_plugin_external_show_configure           (XfcePanelPluginProvider           *provider);
+static gboolean     panel_plugin_external_get_show_about           (XfcePanelPluginProvider           *provider);
+static void         panel_plugin_external_show_about               (XfcePanelPluginProvider           *provider);
+static void         panel_plugin_external_removed                  (XfcePanelPluginProvider           *provider);
+static gboolean     panel_plugin_external_remote_event             (XfcePanelPluginProvider           *provider,
+                                                                    const gchar                       *name,
+                                                                    const GValue                      *value,
+                                                                    guint                             *handler_id);
+static void         panel_plugin_external_set_locked               (XfcePanelPluginProvider           *provider,
+                                                                    gboolean                           locked);
+static void         panel_plugin_external_set_sensitive            (PanelPluginExternal               *external);
+static void         panel_plugin_external_child_watch              (GPid                               pid,
+                                                                    gint                               status,
+                                                                    gpointer                           user_data);
+static void         panel_plugin_external_child_watch_destroyed    (gpointer                           user_data);
 
 
 
@@ -159,6 +161,8 @@ enum
 enum
 {
   SET,
+  REMOTE_EVENT,
+  REMOTE_EVENT_RESULT,
   LAST_SIGNAL
 };
 
@@ -177,7 +181,6 @@ static void
 panel_plugin_external_class_init (PanelPluginExternalClass *klass)
 {
   GObjectClass   *gobject_class;
-  GtkObjectClass *gtkobject_class;
   GtkWidgetClass *gtkwidget_class;
   GtkSocketClass *gtksocket_class;
 
@@ -187,11 +190,9 @@ panel_plugin_external_class_init (PanelPluginExternalClass *klass)
   gobject_class->set_property = panel_plugin_external_set_property;
   gobject_class->get_property = panel_plugin_external_get_property;
 
-  gtkobject_class = GTK_OBJECT_CLASS (klass);
-  gtkobject_class->destroy = panel_plugin_external_destroy;
-
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
   gtkwidget_class->realize = panel_plugin_external_realize;
+  gtkwidget_class->unrealize = panel_plugin_external_unrealize;
 
   gtksocket_class = GTK_SOCKET_CLASS (klass);
   gtksocket_class->plug_removed = panel_plugin_external_plug_removed;
@@ -206,6 +207,24 @@ panel_plugin_external_class_init (PanelPluginExternalClass *klass)
                   G_TYPE_NONE, 1,
                   PANEL_TYPE_DBUS_SET_SIGNAL);
 
+  external_signals[REMOTE_EVENT] =
+    g_signal_new (g_intern_static_string ("remote-event"),
+                  G_TYPE_FROM_CLASS (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  _panel_marshal_VOID__STRING_BOXED_UINT,
+                  G_TYPE_NONE, 3,
+                  G_TYPE_STRING, G_TYPE_VALUE, G_TYPE_UINT);
+
+  external_signals[REMOTE_EVENT_RESULT] =
+    g_signal_new (g_intern_static_string ("remote-event-result"),
+                  G_TYPE_FROM_CLASS (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  _panel_marshal_VOID__UINT_BOOLEAN,
+                  G_TYPE_NONE, 2,
+                  G_TYPE_UINT, G_TYPE_BOOLEAN);
+
   g_object_class_install_property (gobject_class,
                                    PROP_UNIQUE_ID,
                                    g_param_spec_int ("unique-id",
@@ -403,17 +422,6 @@ panel_plugin_external_set_property (GObject      *object,
 
 
 
-static void
-panel_plugin_external_destroy (GtkObject *object)
-{
-  panel_plugin_external_queue_add_noop (PANEL_PLUGIN_EXTERNAL (object),
-                                        TRUE, SIGNAL_WRAPPER_QUIT);
-
-  (*GTK_OBJECT_CLASS (panel_plugin_external_parent_class)->destroy) (object);
-}
-
-
-
 static void
 panel_plugin_external_realize (GtkWidget *widget)
 {
@@ -426,11 +434,14 @@ panel_plugin_external_realize (GtkWidget *widget)
   guint                 argc = PLUGIN_ARGV_ARGUMENTS;
   GPid                  pid;
 
+  panel_return_if_fail (external->pid == 0);
+  panel_return_if_fail (!external->plug_embedded);
+
   /* realize the socket first */
   (*GTK_WIDGET_CLASS (panel_plugin_external_parent_class)->realize) (widget);
 
   /* get the socket id and unique id in a string */
-  socket_id = g_strdup_printf ("%d", gtk_socket_get_id (GTK_SOCKET (widget)));
+  socket_id = g_strdup_printf ("%u", gtk_socket_get_id (GTK_SOCKET (widget)));
   unique_id = g_strdup_printf ("%d", external->unique_id);
 
   /* add the number of arguments to the argc count */
@@ -484,6 +495,31 @@ panel_plugin_external_realize (GtkWidget *widget)
 
 
 
+static void
+panel_plugin_external_unrealize (GtkWidget *widget)
+{
+  PanelPluginExternal *external = PANEL_PLUGIN_EXTERNAL (widget);
+
+  /* the plug is not embedded anymore */
+  external->plug_embedded = FALSE;
+
+  if (external->watch_id != 0)
+    {
+      /* remove the child watch so we don't leave zomies */
+      g_source_remove (external->watch_id);
+      g_child_watch_add (external->pid, (GChildWatchFunc) g_spawn_close_pid, NULL);
+      external->pid = 0;
+    }
+
+  /* quit the plugin */
+  panel_plugin_external_queue_add_noop (PANEL_PLUGIN_EXTERNAL (widget),
+                                        TRUE, SIGNAL_WRAPPER_QUIT);
+
+  (*GTK_WIDGET_CLASS (panel_plugin_external_parent_class)->unrealize) (widget);
+}
+
+
+
 static gboolean
 panel_plugin_external_plug_removed (GtkSocket *socket)
 {
@@ -506,6 +542,7 @@ panel_plugin_external_plug_removed (GtkSocket *socket)
       /* remove the child watch so we don't leave zomies */
       g_source_remove (external->watch_id);
       g_child_watch_add (external->pid, (GChildWatchFunc) g_spawn_close_pid, NULL);
+      external->pid = 0;
     }
 
   window = gtk_widget_get_toplevel (GTK_WIDGET (socket));
@@ -556,26 +593,17 @@ panel_plugin_external_plug_added (GtkSocket *socket)
   /* plug has been added */
   external->plug_embedded = TRUE;
 
+  panel_debug (PANEL_DEBUG_DOMAIN_EXTERNAL,
+               "%d has been embedded, %d values in queue",
+               external->unique_id,
+               external->queue != NULL ? external->queue->len : 0);
+
   if (external->queue != NULL)
     panel_plugin_external_dbus_set (external, FALSE);
 }
 
 
 
-static gboolean
-panel_plugin_external_dbus_reply (PanelPluginExternal  *external,
-                                  guint                 reply_id,
-                                  const GValue         *value,
-                                  GError              **error)
-{
-  panel_return_val_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external), FALSE);
-  panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (external), FALSE);
-
-  return TRUE;
-}
-
-
-
 static gboolean
 panel_plugin_external_dbus_provider_signal (PanelPluginExternal            *external,
                                             XfcePanelPluginProviderSignal   provider_signal,
@@ -606,6 +634,22 @@ panel_plugin_external_dbus_provider_signal (PanelPluginExternal            *exte
 
 
 
+static gboolean
+panel_plugin_external_dbus_remote_event_result (PanelPluginExternal  *external,
+                                                guint                 handle,
+                                                gboolean              result,
+                                                GError              **error)
+{
+  panel_return_val_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external), FALSE);
+
+  g_signal_emit (G_OBJECT (external), external_signals[REMOTE_EVENT_RESULT], 0,
+                 handle, result);
+
+  return TRUE;
+}
+
+
+
 static void
 panel_plugin_external_dbus_set (PanelPluginExternal *external,
                                 gboolean             force)
@@ -636,7 +680,6 @@ panel_plugin_external_queue_add (PanelPluginExternal *external,
 
   panel_return_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external));
   panel_return_if_fail (G_TYPE_CHECK_VALUE (value));
-  panel_return_if_fail (!force || external->queue == NULL);
 
   if (external->queue == NULL)
     external->queue = g_ptr_array_sized_new (1);
@@ -647,7 +690,6 @@ panel_plugin_external_queue_add (PanelPluginExternal *external,
   dbus_g_type_struct_set (&message,
                           DBUS_SET_PROPERTY, property,
                           DBUS_SET_VALUE, value,
-                          DBUS_SET_REPLY_ID, 0,
                           G_MAXUINT);
 
   g_ptr_array_add (external->queue, g_value_get_boxed (&message));
@@ -836,26 +878,34 @@ panel_plugin_external_removed (XfcePanelPluginProvider *provider)
 static gboolean
 panel_plugin_external_remote_event (XfcePanelPluginProvider *provider,
                                     const gchar             *name,
-                                    const GValue            *value)
+                                    const GValue            *value,
+                                    guint                   *handle)
 {
-  GValue        noop_value = { 0, };
+  static guint  handle_counter = 0;
+  GValue        dummy_value = { 0, };
   const GValue *real_value = value;
 
   panel_return_val_if_fail (PANEL_IS_PLUGIN_EXTERNAL (provider), TRUE);
   panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider), TRUE);
   panel_return_val_if_fail (value == NULL || G_IS_VALUE (value), FALSE);
 
-  /* TODO handle the return value */
+  if (G_UNLIKELY (handle_counter > G_MAXUINT - 2))
+    handle_counter = 0;
+  *handle = ++handle_counter;
 
   if (value == NULL)
     {
-      g_value_init (&noop_value, G_TYPE_UCHAR);
-      g_value_set_uchar (&noop_value, '\0');
-      real_value = &noop_value;
+      /* we send a dummy value over dbus */
+      g_value_init (&dummy_value, G_TYPE_UCHAR);
+      g_value_set_uchar (&dummy_value, '\0');
+      real_value = &dummy_value;
     }
 
-  panel_plugin_external_queue_add (PANEL_PLUGIN_EXTERNAL (provider),
-                                   FALSE, name, real_value);
+  g_signal_emit (G_OBJECT (provider), external_signals[REMOTE_EVENT], 0,
+                 name, real_value, *handle);
+
+  if (real_value != value)
+    g_value_unset (&dummy_value);
 
   return TRUE;
 }
@@ -918,12 +968,22 @@ panel_plugin_external_child_watch (GPid     pid,
       switch (WEXITSTATUS (status))
         {
         case PLUGIN_EXIT_SUCCESS:
+          break;
+
         case PLUGIN_EXIT_FAILURE:
         case PLUGIN_EXIT_PREINIT_FAILED:
         case PLUGIN_EXIT_CHECK_FAILED:
         case PLUGIN_EXIT_NO_PROVIDER:
-          /* wait until everything is settled, then destroy the
-           * external plugin so it is removed from the configuration */
+          panel_debug (PANEL_DEBUG_DOMAIN_EXTERNAL,
+                       "Plugin exited with status %d. Removing from "
+                       "configuration.", WEXITSTATUS (status));
+
+          /* cleanup the plugin configuration (in panel-application) */
+          xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (external),
+                                                  PROVIDER_SIGNAL_REMOVE_PLUGIN);
+
+
+          /* wait until everything is settled */
           exo_gtk_object_destroy_later (GTK_OBJECT (external));
           break;
         }
