@@ -146,9 +146,6 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   dialog->bindings = NULL;
   dialog->application = panel_application_get ();
 
-  /* block all autohides */
-  panel_application_windows_autohide (dialog->application, TRUE);
-
   /* load the builder data into the object */
   gtk_builder_add_from_string (GTK_BUILDER (dialog), panel_preferences_dialog_ui,
                                panel_preferences_dialog_ui_length, NULL);
@@ -243,9 +240,6 @@ panel_preferences_dialog_finalize (GObject *object)
           dialog->items_changed_handler_id);
     }
 
-  /* thaw all autohide blocks */
-  panel_application_windows_autohide (dialog->application, FALSE);
-
   /* deselect all windows */
   if (!panel_item_dialog_visible ())
     panel_application_window_select (dialog->application, NULL);
@@ -347,6 +341,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   panel_preferences_dialog_bindings_add (dialog, "autohide", "active");
   panel_preferences_dialog_bindings_add (dialog, "size", "value");
   panel_preferences_dialog_bindings_add (dialog, "length", "value");
+  panel_preferences_dialog_bindings_add (dialog, "length-adjust", "active");
   panel_preferences_dialog_bindings_add (dialog, "background-alpha", "value");
   panel_preferences_dialog_bindings_add (dialog, "enter-opacity", "value");
   panel_preferences_dialog_bindings_add (dialog, "leave-opacity", "value");
