@@ -139,6 +139,7 @@ enum
   PROP_HORIZONTAL,
   PROP_SIZE,
   PROP_LENGTH,
+  PROP_LENGTH_ADJUST,
   PROP_POSITION_LOCKED,
   PROP_AUTOHIDE,
   PROP_SPAN_MONITORS,
@@ -248,6 +249,7 @@ struct _PanelWindow
   /* window positioning */
   guint                size;
   gdouble              length;
+  guint                length_adjust : 1;
   guint                horizontal : 1;
   SnapPosition         snap_position;
   guint                span_monitors : 1;
@@ -341,6 +343,12 @@ panel_window_class_init (PanelWindowClass *klass)
                                                       1, 100, 25,
                                                       EXO_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_LENGTH_ADJUST,
+                                   g_param_spec_boolean ("length-adjust", NULL, NULL,
+                                                         TRUE,
+                                                         EXO_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class,
                                    PROP_POSITION_LOCKED,
                                    g_param_spec_boolean ("position-locked", NULL, NULL,
@@ -413,6 +421,7 @@ panel_window_init (PanelWindow *window)
   window->horizontal = TRUE;
   window->size = 30;
   window->length = 0.10;
+  window->length_adjust = TRUE;
   window->snap_position = SNAP_POSITION_NONE;
   window->span_monitors = FALSE;
   window->position_locked = FALSE;
@@ -469,6 +478,10 @@ panel_window_get_property (GObject    *object,
       g_value_set_uint (value,  rint (window->length * 100.00));
       break;
 
+    case PROP_LENGTH_ADJUST:
+      g_value_set_boolean (value, window->length_adjust);
+      break;
+
     case PROP_POSITION_LOCKED:
       g_value_set_boolean (value, window->position_locked);
       break;
@@ -562,6 +575,15 @@ panel_window_set_property (GObject      *object,
         }
       break;
 
+    case PROP_LENGTH_ADJUST:
+      val_bool = g_value_get_boolean (value);
+      if (window->length_adjust != val_bool)
+        {
+          window->length_adjust = !!val_bool;
+          gtk_widget_queue_resize (GTK_WIDGET (window));
+        }
+      break;
+
     case PROP_POSITION_LOCKED:
       val_bool = g_value_get_boolean (value);
       if (window->position_locked != val_bool)
@@ -1033,11 +1055,17 @@ panel_window_size_request (GtkWidget      *widget,
   /* respect the length and monitor/screen size */
   if (window->horizontal)
     {
+      if (!window->length_adjust)
+        requisition->width = extra_width;
+
       length = window->area.width * window->length;
       requisition->width = CLAMP (requisition->width, length, window->area.width);
     }
   else
     {
+      if (!window->length_adjust)
+        requisition->height = extra_height;
+
       length = window->area.height * window->length;
       requisition->height = CLAMP (requisition->height, length, window->area.height);
     }
@@ -1964,6 +1992,7 @@ panel_window_set_autohide (PanelWindow *window,
                             "resizable", TRUE,
                             "type-hint", GDK_WINDOW_TYPE_HINT_DOCK,
                             "gravity", GDK_GRAVITY_STATIC,
+                            "name", "XfcePanelWindowHidden",
                             NULL);
 
       /* move the window offscreen */
@@ -2170,9 +2199,11 @@ panel_window_update_plugins (PanelWindow *window,
     }
 
   itembar = gtk_bin_get_child (GTK_BIN (window));
-  panel_return_if_fail (GTK_IS_CONTAINER (itembar));
   if (G_LIKELY (itembar != NULL))
-    gtk_container_foreach (GTK_CONTAINER (itembar), func, window);
+    {
+      panel_return_if_fail (GTK_IS_CONTAINER (itembar));
+      gtk_container_foreach (GTK_CONTAINER (itembar), func, window);
+    }
 }
 
 
@@ -2285,6 +2316,7 @@ panel_window_new (void)
                        "resizable", FALSE,
                        "type-hint", GDK_WINDOW_TYPE_HINT_DOCK,
                        "gravity", GDK_GRAVITY_STATIC,
+                       "name", "XfcePanelWindow",
                        NULL);
 }
 
@@ -2297,10 +2329,6 @@ panel_window_set_povider_info (PanelWindow *window,
   panel_return_if_fail (PANEL_IS_WINDOW (window));
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider));
 
-  panel_window_set_plugin_size (provider, window);
-  panel_window_set_plugin_orientation (provider, window);
-  panel_window_set_plugin_screen_position (provider, window);
-
   xfce_panel_plugin_provider_set_locked (XFCE_PANEL_PLUGIN_PROVIDER (provider),
                                          panel_window_get_locked (window));
 
@@ -2313,6 +2341,10 @@ panel_window_set_povider_info (PanelWindow *window,
         panel_plugin_external_46_set_background_alpha (PANEL_PLUGIN_EXTERNAL_46 (provider),
             PANEL_BASE_WINDOW (window)->background_alpha);
     }
+
+  panel_window_set_plugin_orientation (provider, window);
+  panel_window_set_plugin_screen_position (provider, window);
+  panel_window_set_plugin_size (provider, window);
 }
 
 
