@@ -53,9 +53,11 @@ struct _ApplicationsMenuPlugin
   GtkWidget       *box;
   GtkWidget       *icon;
   GtkWidget       *label;
+  GtkWidget       *menu;
 
   guint            show_generic_names : 1;
   guint            show_menu_icons : 1;
+  guint            show_tooltips : 1;
   guint            show_button_title : 1;
   gchar           *button_title;
   gchar           *button_icon;
@@ -72,6 +74,7 @@ enum
   PROP_0,
   PROP_SHOW_GENERIC_NAMES,
   PROP_SHOW_MENU_ICONS,
+  PROP_SHOW_TOOLTIPS,
   PROP_SHOW_BUTTON_TITLE,
   PROP_BUTTON_TITLE,
   PROP_BUTTON_ICON,
@@ -79,6 +82,10 @@ enum
   PROP_CUSTOM_MENU_FILE
 };
 
+static const GtkTargetEntry dnd_target_list[] = {
+  { "text/uri-list", 0, 0 }
+};
+
 
 
 static void      applications_menu_plugin_get_property         (GObject                *object,
@@ -144,6 +151,12 @@ applications_menu_plugin_class_init (ApplicationsMenuPluginClass *klass)
                                                          NULL, NULL,
                                                          TRUE,
                                                          EXO_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_SHOW_TOOLTIPS,
+                                   g_param_spec_boolean ("show-tooltips",
+                                                         NULL, NULL,
+                                                         FALSE,
+                                                         EXO_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_SHOW_BUTTON_TITLE,
@@ -239,12 +252,16 @@ applications_menu_plugin_get_property (GObject    *object,
       g_value_set_boolean (value, plugin->show_menu_icons);
       break;
 
+    case PROP_SHOW_TOOLTIPS:
+      g_value_set_boolean (value, plugin->show_tooltips);
+      break;
+
     case PROP_SHOW_BUTTON_TITLE:
       g_value_set_boolean (value, plugin->show_button_title);
       break;
 
     case PROP_BUTTON_TITLE:
-      g_value_set_string (value, exo_str_is_empty (plugin->button_title) ?
+      g_value_set_string (value, plugin->button_title == NULL ?
           DEFAULT_TITLE : plugin->button_title);
       break;
 
@@ -287,6 +304,10 @@ applications_menu_plugin_set_property (GObject      *object,
       plugin->show_menu_icons = g_value_get_boolean (value);
       break;
 
+    case PROP_SHOW_TOOLTIPS:
+      plugin->show_tooltips = g_value_get_boolean (value);
+      break;
+
     case PROP_SHOW_BUTTON_TITLE:
       plugin->show_button_title = g_value_get_boolean (value);
       if (plugin->show_button_title)
@@ -302,7 +323,8 @@ applications_menu_plugin_set_property (GObject      *object,
       plugin->button_title = g_value_dup_string (value);
       gtk_label_set_text (GTK_LABEL (plugin->label),
           plugin->button_title != NULL ? plugin->button_title : "");
-      gtk_widget_set_tooltip_text (plugin->button, plugin->button_title);
+      gtk_widget_set_tooltip_text (plugin->button,
+          exo_str_is_empty (plugin->button_title) ? NULL : plugin->button_title);
       break;
 
     case PROP_BUTTON_ICON:
@@ -338,6 +360,7 @@ applications_menu_plugin_construct (XfcePanelPlugin *panel_plugin)
     { "show-generic-names", G_TYPE_BOOLEAN },
     { "show-menu-icons", G_TYPE_BOOLEAN },
     { "show-button-title", G_TYPE_BOOLEAN },
+    { "show-tooltips", G_TYPE_BOOLEAN },
     { "button-title", G_TYPE_STRING },
     { "button-icon", G_TYPE_STRING },
     { "custom-menu", G_TYPE_BOOLEAN },
@@ -469,6 +492,9 @@ applications_menu_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
   ApplicationsMenuPlugin *plugin = XFCE_APPLICATIONS_MENU_PLUGIN (panel_plugin);
   GtkBuilder             *builder;
   GObject                *dialog, *object, *object2;
+  guint                   i;
+  const gchar            *check_names[] = { "show-generic-names", "show-menu-icons",
+                                            "show-tooltips", "show-button-title" };
 
   /* setup the dialog */
   PANEL_UTILS_LINK_4UI
@@ -477,20 +503,13 @@ applications_menu_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
   if (G_UNLIKELY (builder == NULL))
     return;
 
-  object = gtk_builder_get_object (builder, "show-generic-names");
-  panel_return_if_fail (GTK_IS_CHECK_BUTTON (object));
-  exo_mutual_binding_new (G_OBJECT (plugin), "show-generic-names",
-                          G_OBJECT (object), "active");
-
-  object = gtk_builder_get_object (builder, "show-menu-icons");
-  panel_return_if_fail (GTK_IS_CHECK_BUTTON (object));
-  exo_mutual_binding_new (G_OBJECT (plugin), "show-menu-icons",
-                          G_OBJECT (object), "active");
-
-  object = gtk_builder_get_object (builder, "show-button-title");
-  panel_return_if_fail (GTK_IS_CHECK_BUTTON (object));
-  exo_mutual_binding_new (G_OBJECT (plugin), "show-button-title",
-                          G_OBJECT (object), "active");
+  for (i = 0; i < G_N_ELEMENTS (check_names); ++i)
+    {
+      object = gtk_builder_get_object (builder, check_names[i]);
+      panel_return_if_fail (GTK_IS_CHECK_BUTTON (object));
+      exo_mutual_binding_new (G_OBJECT (plugin), check_names[i],
+                              G_OBJECT (object), "active");
+    }
 
   object = gtk_builder_get_object (builder, "button-title");
   panel_return_if_fail (GTK_IS_ENTRY (object));
@@ -572,6 +591,7 @@ applications_menu_plugin_menu_deactivate (GtkWidget *menu,
   panel_return_if_fail (button == NULL || GTK_IS_TOGGLE_BUTTON (button));
   panel_return_if_fail (GTK_IS_MENU (menu));
 
+  /* button is NULL when we popup the menu under the cursor position */
   if (button != NULL)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
 
@@ -689,6 +709,55 @@ applications_menu_plugin_menu_item_activate (GtkWidget      *mi,
 
 
 
+static void
+applications_menu_plugin_menu_item_drag_begin (GarconMenuItem   *item,
+                                               GdkDragContext   *drag_context)
+{
+  const gchar *icon_name;
+
+  panel_return_if_fail (GARCON_IS_MENU_ITEM (item));
+
+  icon_name = garcon_menu_item_get_icon_name (item);
+  if (!exo_str_is_empty (icon_name))
+    gtk_drag_set_icon_name (drag_context, icon_name, 0, 0);
+}
+
+
+
+static void
+applications_menu_plugin_menu_item_drag_data_get (GarconMenuItem   *item,
+                                                  GdkDragContext   *drag_context,
+                                                  GtkSelectionData *selection_data,
+                                                  guint             info,
+                                                  guint             drag_time)
+{
+  gchar *uris[2] = { NULL, NULL };
+
+  panel_return_if_fail (GARCON_IS_MENU_ITEM (item));
+
+  uris[0] = garcon_menu_item_get_uri (item);
+  if (G_LIKELY (uris[0] != NULL))
+    {
+      gtk_selection_data_set_uris (selection_data, uris);
+      g_free (uris[0]);
+    }
+}
+
+
+
+static void
+applications_menu_plugin_menu_item_drag_end (ApplicationsMenuPlugin *plugin)
+{
+  panel_return_if_fail (XFCE_IS_APPLICATIONS_MENU_PLUGIN (plugin));
+  panel_return_if_fail (GTK_IS_TOGGLE_BUTTON (plugin->button));
+  panel_return_if_fail (GTK_IS_MENU (plugin->menu));
+
+  /* selection-done is never called, so handle that manually */
+  applications_menu_plugin_menu_deactivate (plugin->menu, plugin->button);
+}
+
+
+
 static gboolean
 applications_menu_plugin_menu_add (GtkWidget              *gtk_menu,
                                    GarconMenu             *menu,
@@ -697,6 +766,7 @@ applications_menu_plugin_menu_add (GtkWidget              *gtk_menu,
   GList               *elements, *li;
   GtkWidget           *mi, *image;
   const gchar         *name, *icon_name;
+  const gchar         *comment;
   GtkWidget           *submenu;
   gboolean             has_children = FALSE;
   gint                 size = DEFAULT_ICON_SIZE, w, h;
@@ -729,14 +799,33 @@ applications_menu_plugin_menu_add (GtkWidget              *gtk_menu,
 
           mi = gtk_image_menu_item_new_with_label (name);
           gtk_menu_shell_append (GTK_MENU_SHELL (gtk_menu), mi);
-
           g_signal_connect_data (G_OBJECT (mi), "activate",
               G_CALLBACK (applications_menu_plugin_menu_item_activate),
               g_object_ref (li->data), (GClosureNotify) g_object_unref, 0);
           gtk_widget_show (mi);
 
+          if (plugin->show_tooltips)
+            {
+              comment = garcon_menu_item_get_comment (li->data);
+              if (!exo_str_is_empty (comment))
+                gtk_widget_set_tooltip_text (mi, comment);
+            }
+
+          /* dragging items from the menu to the panel */
+          gtk_drag_source_set (mi, GDK_BUTTON1_MASK, dnd_target_list,
+              G_N_ELEMENTS (dnd_target_list), GDK_ACTION_COPY);
+          g_signal_connect_data (G_OBJECT (mi), "drag-begin",
+              G_CALLBACK (applications_menu_plugin_menu_item_drag_begin),
+              g_object_ref (li->data), (GClosureNotify) g_object_unref, G_CONNECT_SWAPPED);
+          g_signal_connect_data (G_OBJECT (mi), "drag-data-get",
+              G_CALLBACK (applications_menu_plugin_menu_item_drag_data_get),
+              g_object_ref (li->data), (GClosureNotify) g_object_unref, G_CONNECT_SWAPPED);
+          g_signal_connect_swapped (G_OBJECT (mi), "drag-end",
+              G_CALLBACK (applications_menu_plugin_menu_item_drag_end), plugin);
+
           command = garcon_menu_item_get_command (li->data);
-          gtk_widget_set_sensitive (mi, !exo_str_is_empty (command));
+          if (G_UNLIKELY (exo_str_is_empty (command)))
+            gtk_widget_set_sensitive (mi, FALSE);
 
           if (plugin->show_menu_icons)
             {
@@ -765,7 +854,7 @@ applications_menu_plugin_menu_add (GtkWidget              *gtk_menu,
            * return value of this function, so avoid that and only check
            * the visibility of the menu directory */
           directory = garcon_menu_get_directory (li->data);
-          if (!garcon_menu_directory_get_visible (directory))
+          if (directory != NULL && !garcon_menu_directory_get_visible (directory))
             continue;
 
           submenu = gtk_menu_new ();
@@ -809,7 +898,7 @@ static void
 applications_menu_plugin_menu (GtkWidget              *button,
                                ApplicationsMenuPlugin *plugin)
 {
-  GtkWidget  *gtk_menu, *mi;
+  GtkWidget  *mi;
   GarconMenu *menu;
   GError     *error = NULL;
 
@@ -828,19 +917,20 @@ applications_menu_plugin_menu (GtkWidget              *button,
 
   if (garcon_menu_load (menu, NULL, &error))
     {
-      gtk_menu = gtk_menu_new ();
-      g_signal_connect (G_OBJECT (gtk_menu), "deactivate",
+      plugin->menu = gtk_menu_new ();
+      g_signal_connect (G_OBJECT (plugin->menu), "selection-done",
            G_CALLBACK (applications_menu_plugin_menu_deactivate), button);
+      g_object_add_weak_pointer (G_OBJECT (plugin->menu), (gpointer) &plugin->menu);
 
-      if (!applications_menu_plugin_menu_add (gtk_menu, menu, plugin))
+      if (!applications_menu_plugin_menu_add (plugin->menu, menu, plugin))
         {
           mi = gtk_menu_item_new_with_label (_("No applications found"));
-          gtk_menu_shell_append (GTK_MENU_SHELL (gtk_menu), mi);
+          gtk_menu_shell_append (GTK_MENU_SHELL (plugin->menu), mi);
           gtk_widget_set_sensitive (mi, FALSE);
           gtk_widget_show (mi);
         }
 
-      gtk_menu_popup (GTK_MENU (gtk_menu), NULL, NULL,
+      gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
                       button != NULL ? xfce_panel_plugin_position_menu : NULL,
                       plugin, 1, gtk_get_current_event_time ());
     }
